<?php

function lts_is_lts_up_to_date($requirements = array()) {
  return $requirements;
}

function lts_nosupport_disclaimer($requirements = array()) {
  $requirements['drupal5_ns'] = array(
    'title' => 'Drupal 5 No longer Officially Supported',
    'value' => '',
    'description' => t(LTS_SUPPORT_STMT),
    'severity' => REQUIREMENT_WARNING,
  );
  return $requirements;

}

function lts_check_core_patches($requirements = array()) {
  $patches = ltls_get_patches('core_patches');
  foreach ($patches as $i => $patch) {

    if (!$patch['patched']) {
      $tokens = lts_tokenify($patch);
      $id = $patch['patch_file_name'];
      $requirements[$id] = array(
        'value' => $id,
      );
      $tokens['!sa_link'] = l(t('Security Advisory #!advisory', $tokens), 'http://drupal.org/node/'. $patch['advisory']);
      $tokens['!lts_link'] = l(t('LTS Module Issue #!lts_issue', $tokens), 'http://drupal.org/node/'. $patch['lts_issue']);
      $requirements[$id]['description'] = t('This patch is related to !sa_link
        and !lts_link.  The patch itelf is at !patch_path', $tokens);
      if ($patch['not_patched']) { //not patched
        $requirements[$id]['severity'] = REQUIREMENT_ERROR;
        $requirements[$id]['title'] = t('Drupal LTS Patch #!lts_issue not applied.', $tokens);

      }
      else { // maybe patched, maybe not, couldn't tell
        $requirements[$id]['severity'] = REQUIREMENT_WARNING;
        $requirements[$id]['title'] = t('Its unclear if Drupal LTS Patch #!lts_issue has been applied.  LTS Module does not find the old code or the new code in !target_path.', $tokens);
      }

    }

  }
  return $requirements;
}

function lts_tokenify($patch) {
  $tokens = array();
  foreach ($patch as $key => $value) {
    $tokens['!' . $key] = $value;
    $tokens['@' . $key] = $value;
    $tokens['%' . $key] = $value;
  }
  return $tokens;

}


function lts_check_contrib_patches($requirements = array()) {
  return $requirements;
}


function ltls_get_patches($dir = 'core_patches') {

  $path = drupal_get_path('module','lts') ."/$dir/index.csv";
  $handle = fopen($path,'r');
  $patches = array();

  $header = TRUE;
  if (($handle = fopen($path, "r")) !== FALSE) {
    while (($data = fgetcsv($handle, 0, ',', '"')) !== FALSE) {
      if ($header || count($data) < 3) {
        $header = FALSE;
        continue; //skip header
      }

      $patch = array(
        'patch_path' => drupal_get_path('module','lts') ."/$dir/". $data[0],
        'patch_file_name' => $data[0],
        'advisory' => $data[1],
        'lts_issue' => $data[2],
        'description'  => $data[3],
      );
      $patch = lts_parse_patch($patch);
      $target_file_contents = file_get_contents($patch['target_path']);
      $patch['patched'] = (boolean)strpos(lts_cleanse($target_file_contents), lts_cleanse($patch['new_code'])) > 0;
      $patch['not_patched'] = (boolean)strpos(lts_cleanse($target_file_contents), lts_cleanse($patch['old_code'])) > 0;


      $patches[] = $patch;
    }
  }
  fclose($handle);

  return $patches;

}

function lts_cleanse($str) {
  return str_replace(array(' ',"\n", "\r"), array('','',''), $str);
}

function lts_parse_patch($patch) {

  $old_code = array();
  $new_code = array();
  $patch_body = array();
  $code_part_of_patch = FALSE;
  $line_i = 0;
  if (($handle = fopen($patch['patch_path'], "r")) !== FALSE) {
    while (($line = fgets($handle)) !== FALSE) {
     // print $line;
      if (trim($line) == "" && $code_part_of_patch == FALSE) {
        $code_part_of_patch = TRUE;
        continue;
      }
      if (!$code_part_of_patch) {
        if ($line_i == 0) {
          $parts = explode(' ', $line);
          $patch['target_path'] = trim($parts[1]);
        }
      }
      else {
        if (strpos($line, '-') === 0) {
          $old_code[] = str_replace('-', ' ', $line);

        }
        elseif (strpos($line, '+') === 0) {
          $new_code[] = str_replace('+', ' ', $line);

        }
        else {
          $old_code[] = $line;
          $new_code[] = $line;
        }
        $patch_body[] = $line;
      }
      $line_i = $line_i + 1;
    }
    $patch['old_code'] = join(NULL , $old_code);
    $patch['new_code'] = join(NULL, $new_code);
    $patch['patch_body'] = join(NULL, $patch_body);

  }

  fclose($handle);

  return $patch;

}
